#!/bin/bash

. /usr/local/Reductor/etc/const
. $VENVBIN/activate
. $CONFIG

set -eu

LOCKFILE_URL=$LOCKDIR/url.lock
LOCKFILE_DOMAIN=$LOCKDIR/domain.lock
LOCKFILE_IP=$LOCKDIR/ip.lock
LOCKFILE_PORT=$LOCKDIR/port.lock

FD_URL=3
FD_DOMAIN=4
FD_IP=5
FD_PORT=6

cleanup() {
	local prefix
	local actual=$LISTDIR/tmp/rkn_prefix_list_actual
	local current=$LISTDIR/tmp/rkn_prefix_list_current
	local delete=$LISTDIR/tmp/rkn_prefix_list_delete
	local bin=${BINDIR}/dump_parser.py
	mkdir -p $LISTDIR/{rkn,tmp}/
	mkdir -p $LISTDIR/rkn_archive/{00..23}/
	[ -d /cfg ] && bin="${bin}c"
	python "$bin" prefix_list | sort -u > $actual
	find $LISTDIR/rkn/ -type f | sed 's|.*/rkn.||g' | sort -u > $current
	join -a 1 -v 1 -j 1 $current $actual > $delete
	while read prefix; do
		rm -vf $LISTDIR/rkn/rkn.$prefix
	done < $delete
}

main() {
	local dump="${1:-$DUMPXML}"
	local lockfile
	local bin=${BINDIR}/dump_parser.py
	[ ! -f "$dump" ] && exit 0
	[ ! -s "$dump" ] && log "WARNING: empty dump.xml" && exit 0
	if ! declare -p filter | grep -q '^declare -A'; then
		declare -A filter
	fi
	BLOCK_HTTPS_IP_EVEN_IF_HTTP="${filter['https_by_ip_on_http']:-0}"
	export BLOCK_HTTPS_IP_EVEN_IF_HTTP
	mkdir -p $LOCKDIR
	for list_type in URL DOMAIN IP PORT; do
		lockfile=LOCKFILE_$list_type
		fd=FD_$list_type
		set_lock ${!lockfile} 60 ${!fd}
	done
	cleanup
	[ -d /cfg ] && bin="${bin}c"
	python "$bin" "$dump" writer
	if [ "${filter['keep_rkn_archive']:-0}" == 1 ]; then
		log "Сохраняем предыдущие списки на 24 часа."
		rsync -a --delete "$LISTDIR/rkn/" "$LISTDIR/rkn_archive/$(date +%H)"
		# Нужно на случай если допустим в 3 часа дня выгрузка не
		# работала неделю, списки не должны блокироваться неделю.
		find $LISTDIR/rkn_archive/ -mtime +0 -type f -delete
	else
		find $LISTDIR/rkn_archive/ -type f -delete
	fi
	for list_type in URL DOMAIN IP PORT; do
		lockfile=LOCKFILE_$list_type
		fd=FD_$list_type
		release_lock ${!lockfile} ${!fd}
	done
}

main "$@"
