#!/bin/bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. $CONFIG

set -eu

HTTP_PROCFILE=/proc/net/ipt_reductor/block_list
HTTP_LOCKFILE=$LOCKDIR/ipt_reductor.lock
HTTP_FD=10

DNS_PROCFILE=/proc/net/ipt_dnsmatch/block_list
DNS_LOCKFILE=$LOCKDIR/ipt_dnsmatch.lock
DNS_FD=11

SNI_PROCFILE=/proc/net/ipt_snimatch/block_list
SNI_LOCKFILE=$LOCKDIR/ipt_snimatch.lock
SNI_FD=12

LOCKFILE_URL=$LOCKDIR/url.lock
FD_URL=3

LOCKFILE_DOMAIN=$LOCKDIR/domain.lock
FD_DOMAIN=4

CUSTOM_EXIT=1

__custom_exit() {
	release_lock $HTTP_LOCKFILE $HTTP_FD &>/dev/null || true
	release_lock $DNS_LOCKFILE $DNS_FD &>/dev/null || true
	release_lock $SNI_LOCKFILE $SNI_FD &>/dev/null || true
	release_lock $LOCKFILE_URL $FD_URL &>/dev/null || true
	release_lock $LOCKFILE_DOMAIN $FD_DOMAIN &>/dev/null || true
}

trap __exit EXIT

prepare() {
	mkdir -p $TMPDIR $LOCKDIR
}

log_module() {
	local module="$1"
	local state="$2"
	while read -r line; do
		log "$module.$state.$line"
	done <<< "$(egrep "(database|state)" /proc/net/$module/block_list || true)"
}

# теперь можем спокойно использовать этот враппер везде, скорость load НЕ ВАЖНА из-за double buffer
write2proc() {
	echo "$@" > $HTTP_PROCFILE
}

writable() {
	local state=0
	local module="$1"
	$BINDIR/modules_ctl get_state $module || state=$?
	if [ "$state" != 0 ]; then
		log "Модуль $module не доступен для записи (state=$state)."
		return 1
	fi
}

load_url_and_signatures_to_proc() {
	local failed=0
	local list_id="$1"
	local input_file="$2"
	echo >> $LOGFILE || true
	log "INFO: начинаем загрузку URL в ядро" >> $LOGFILE || true
	while read sign url; do
		if ! write2proc "$sign $list_id $url"; then
			log "ERROR: не удалось добавить _ $sign $list_id $url _"
			((failed++)) || true
		fi
	done < "$input_file"
	log "Загрузка URL в ядро закончена" >> $LOGFILE || true
	[ "$failed" -gt '0' ] && log "ERROR: $failed URL не было добавлено" | tee -a $LOGFILE || true
	echo >> $LOGFILE || true
	return 0
}

load_signatures_and_cache() {
	local line
	local signatures=$CACHEDIR/signatures.cache
	local loadfile=$TMPDIR/signatures_and_urls.tmp
	set_lock $HTTP_LOCKFILE 60 $HTTP_FD 0 || return 1
	set_lock $LOCKFILE_URL 60 $FD_URL 0 || return 1
	writable ipt_reductor || return 1
	log "Заливаем URL в HTTP-матчер"
	log_module ipt_reductor before
	write2proc clear
	for list in $(url_lists); do
		log "  reductor <- $list"
		# -t <TAB>, not a few spaces
		if ! LC_ALL='' LC_CTYPE='' LANG='' join -t '	' -1 1 -2 2 -o "2.1 1.1" $list $signatures > $loadfile; then
			if [ "${autoupdate['enabled']}" = '0' ] && [ "$list" = "$LISTDIR/load/url_http.load" ] && [ ! -f "$list" ]; then
				log "Список $list отсутствует, автообновление отключено, так что пропускаем загрузку URL."
				log "Предположительно Carbon Reductor используется в качестве редиректора абонентов с отрицательным балансом и ему не нужен URL Matching."
				continue
			fi
			log "WARNING: Ошибка при обработке списка $list, пробуем подождать пару секунд и повторить..." | tee -a $LOGFILE || true
			log "ERROR: LC_ALL='' LC_CTYPE='' LANG='' join -t '      ' -1 1 -2 2 -o '2.1 1.1' $list $signatures > $loadfile"
			log "WARNING: Пытаемся обработать списки, мы обязаны любым способом запуститься с загруженными списками."
			release_lock $LOCKFILE_URL $FD_URL
			nice -n 19 $BINDIR/prepare_lists.sh
			set_lock $LOCKFILE_URL 60 $FD_URL 0 || continue
			sync
			log "Повторная попытка загрузки:"
			log "  $list"
			if ! LC_ALL='' LC_CTYPE='' LANG='' join -t '	' -1 1 -2 2 -o "2.1 1.1" $list $signatures > $loadfile; then
				log "ERROR: Ошибка при обработке списка $list" | tee -a $LOGFILE || true
				continue
			fi
		fi
		grep -v ^clear $TMPDIR/signatures_and_urls.tmp > $TMPDIR/signatures_and_urls || true
		list_id=${list##*/}
		[ "$list_id" = "url_http.load" ] && list_id=0 || true
		load_url_and_signatures_to_proc $list_id $TMPDIR/signatures_and_urls
	done
	release_lock $LOCKFILE_URL $FD_URL
	write2proc switch
	log_module ipt_reductor after
	release_lock $HTTP_LOCKFILE $HTTP_FD
}

use_hook() {
	hook="$HOOKDIR/${0##*/}"
	# shellcheck disable=SC1090
	[ ! -x "$hook" ] || . "$hook"
}

client_prepare_hook() {
	return 0
}

prepare_lists_if_need() {
	local empty_url_load=0
	local empty_rkn=0
	# два раза проверка чтобы не ждать лишний раз захвата лока.
	if [ -s $DUMPXML ] && [ ! -s $LISTDIR/rkn/rkn.url_http ]; then
		set_lock $LOCKFILE_URL 60 $FD_URL 0 || return 1
		[ ! -s $LISTDIR/rkn/rkn.url_http ] && empty_rkn=1
		release_lock $LOCKFILE_URL $FD_URL
		if [ ! -s $LISTDIR/rkn/rkn.ip_https ]; then
			log "Разбираем единый реестр запрещённых сайтов, так-как один из необходимых списков пустой."
			$BINDIR/support_info.sh lists_size | grep rkn | while IFS= read line; do log "$line"; done
			nice -n 19 $BINDIR/dump_parser.sh
		fi
	fi
	# два раза проверка чтобы не ждать лишний раз захвата лока.
	if [ "${autoupdate['enabled']}" = '1' ] && [ ! -s $LISTDIR/load/url_http.load ]; then
		set_lock $LOCKFILE_URL 60 $FD_URL 0 || return 1
		[ ! -s $LISTDIR/load/url_http.load ] && empty_url_load=1
		release_lock $LOCKFILE_URL $FD_URL
		if [ "$empty_url_load" = '1' ]; then
			log "Агрегируем списки, так как url_http.load пустой"
			$BINDIR/support_info.sh lists_size | grep load | while IFS= read line; do log "$line"; done
			nice -n 19 $BINDIR/prepare_lists.sh
		fi
	fi
}

load_sni_list() {
	local tmpdir="$1"
	local prefix="$2"
	local id="$3"
	local list_load="$tmpdir/$prefix.load"
	local list_orig="$LISTDIR/load/$prefix.load"
	local bin="$BINDIR/sniload.py"
	log "  snimatch <- $list_orig"
	if ! [ -s "$list_load" -a -s "$list_orig" ]; then
		log "WARNING: Пустой $list_orig / $list_load"
		return 0
	fi
	[ -d /cfg ] && bin=${bin}c  # if python sources wiped away
	python -u "$bin" "$list_load" "$id" | while read line; do
		echo "$line" > $SNI_PROCFILE
	done
}

load_sni_use_list() {
	local tmpdir="$1"
	local prefix="$2"
	local list="$LISTDIR/load/$prefix.load"
	[ -s "$list" ] && cp -a "$list" "$tmpdir"
}

load_sni() {
	local prefix
	local tmpdir="$TMPDIR/load_sni.$$"
	$BINDIR/dnsenabled --sni || return 0
	writable ipt_snimatch || return 1
	log "Загрузка доменов в snimatch"

	set_lock $LOCKFILE_DOMAIN 60 $FD_DOMAIN 0 || return 1
	# копируем списки перед их использованием, чтобы продержать domain_lock меньше времени
	mkdir -p "$tmpdir"
	find "$tmpdir" -type f -delete
	load_sni_use_list "$tmpdir" domain_mask
	if $BINDIR/dnsenabled --separate-exact-and-mask; then
		load_sni_use_list "$tmpdir" domain_exact
	fi
	if $BINDIR/dnsenabled --proxy-https; then
		load_sni_use_list "$tmpdir" domain_proxy
	fi
	release_lock $LOCKFILE_DOMAIN $FD_DOMAIN

	# для SNI не имеет значения id списка, так что всегда 0
	log_module ipt_snimatch before
	set_lock $SNI_LOCKFILE 60 $SNI_FD 0 || return 1

	echo "clear" > $SNI_PROCFILE
	load_sni_list "$tmpdir" domain_mask 0
	if $BINDIR/dnsenabled --separate-exact-and-mask; then
		load_sni_list "$tmpdir" domain_exact 0
	fi
	if $BINDIR/dnsenabled --proxy-https; then
		load_sni_list "$tmpdir" domain_proxy 0
	fi
	echo "switch" > $SNI_PROCFILE
	rm -rf "$tmpdir"
	release_lock $SNI_LOCKFILE $SNI_FD
	log_module ipt_snimatch after
}

load_dns_list() {
	local list="$1"
	local list_id="${2:-0}"
	local exact="${3:-0}"
	local bin=$BINDIR/dnsload.py
	log "  dnsmatch <- $list"
	if [[ "$list" != $LISTDIR/load/domain_*.load ]]; then
		list_id="${list##*/}"
		read _ _ _ exact _ <<< "$(grep "^$list_id " $HOOKDIR/custom_bindings_dns)" || return 0
	fi
	if ! [ -s $list ]; then
		log "WARNING: Пустой $list"
		return 0
	fi
	[ -d /cfg ] && bin=${bin}c  # if python sources wiped away
	python -u "$bin" "$list" "$list_id" "$exact" | while read domain; do
		echo -n "$domain" > $DNS_PROCFILE
	done
}

load_dns() {
	local prefix
	$BINDIR/dnsenabled || return 0
	writable ipt_dnsmatch || return 1
	set_lock $DNS_LOCKFILE 60 $DNS_FD 0 || return 1
	set_lock $LOCKFILE_DOMAIN 60 $FD_DOMAIN 0 || return 1
	log "Загрузка доменов в dnsmatch"
	log_module ipt_dnsmatch before
	echo "clear" > $DNS_PROCFILE
	load_dns_list $LISTDIR/load/domain_mask.load 0 0
	if $BINDIR/dnsenabled --separate-exact-and-mask; then
		load_dns_list $LISTDIR/load/domain_exact.load 1000 1
	fi
	if $BINDIR/dnsenabled --proxy-https; then
		load_dns_list $LISTDIR/load/domain_proxy.load 2000 0
	fi
	if $BINDIR/dnsenabled --proxy-hsts; then
		load_dns_list $LISTDIR/load/domain_hsts.load 3000 0
	fi
	#TODO: тестировать
	for list in $(domain_custom_lists); do
		load_dns_list $list
	done
	release_lock $LOCKFILE_DOMAIN $FD_DOMAIN
	echo "switch" > $DNS_PROCFILE
	log_module ipt_dnsmatch after
	release_lock $DNS_LOCKFILE $DNS_FD
}

main() {
	local retval=0
	local maintain=0
	prepare
	prepare_lists_if_need
	client_prepare_hook
	if $BINDIR/dnsenabled --revisor && ! $BINDIR/maintain.sh is_locked; then
		maintain=1
		log "На время загрузки списков блокируем доступ в интернет ревизору. Временное решение редкой плавающей проблемы."
		$BINDIR/maintain.sh lock
	fi
	$BINDIR/load_ip.sh
	load_signatures_and_cache || retval=$?
	load_dns || retval=$?
	load_sni || retval=$?
	[ "$maintain" = '1' ] && $BINDIR/maintain.sh unlock
	return $retval
}

use_hook
main "$@"
