#!/bin/bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. "$CONFIG"

set -eu
trap __exit EXIT

LOCKFILE_IP_LOAD=$LOCKDIR/ipset_load.lock
LOCKFILE_IP=$LOCKDIR/ip.lock
LOCKFILE_PORT=$LOCKDIR/port.lock

FD_IP_LOAD=5
FD_IP=6
FD_PORT=7

file2ipset_safe() {
	local ipset=$1
	local file=$2
	ipset flush "$ipset"
	while read item _; do
		ipset add "$ipset" "$item" || true
	done < "$file"
}

file2ipset_fast() {
	local ipset=$1
	local file=$2
	ipset save "$ipset" | grep ^create | sed 's/create /ipset &_/g' | bash
	ipset flush "_$ipset"
	grep -v ^$ "$file" | sed -e "s/^/add _$ipset /" | sort -u | ipset restore
	ipset swap "_$ipset" "$ipset"
	ipset flush "_$ipset"
	ipset destroy "_$ipset"
}

file2ipset() {
	# local setname="$1"
	local filename="$2"
	local silent="${3:-0}"
	if [ ! -f "$filename" ]; then
		[ "$silent" = '1' ] || log "Пропускаем загрузку пустого $filename"
		return 0
	fi
	file2ipset_fast "$@" || file2ipset_safe "$@"
}

load_ip_whitelists() {
	local list
	local silent=1
	for proto in block https; do
		list=$LISTDIR/provider/our.ip_${proto}_whitelist
		[ ! -f "$list" ] || file2ipset ip_${proto}_whitelist "$list" "$silent"
	done
}

load_ip() {
	local prefix list
	set_lock "$LOCKFILE_IP" 60 "$FD_IP" 1
	load_ip_whitelists
	for prefix in ip_https ip_https_plus ip_block; do
		list="$LISTDIR/load/$prefix.load"
		[ "$prefix" = 'ip_block' ] && list="$LISTDIR/load/ip_subnet_block.load"
		# shellcheck disable=SC2154
		log "  $prefix <- $list"
		file2ipset $prefix "$list"
	done
	if [ "${filter['ipv6']:-0}" = '1' ]; then
		for prefix in ip6_https ip6_https_plus; do
			file2ipset "$prefix" "$LISTDIR/load/$prefix.load"
		done
	fi
	release_lock "$LOCKFILE_IP" "$FD_IP"
}

load_ports() {
	local prefix
	set_lock "$LOCKFILE_PORT" 60 "$FD_PORT" 1
	for prefix in port_http port_https; do
		file2ipset $prefix "$LISTDIR/load/$prefix.load"
	done
	ipset test port_http 80 &>/dev/null || ipset add port_http 80
	ipset test port_https 443 &>/dev/null || ipset add port_https 443
	release_lock "$LOCKFILE_PORT" "$FD_PORT"
}

load_ip_ports() {
	[ -s "$LISTDIR/load/ip_port.load" ] || return 0
	set_lock "$LOCKFILE_IP" 60 "$FD_IP" 1
	awk '{print $1}' "$LISTDIR/load/ip_port.load" > "$LISTDIR/tmp/ip_port_ip.$$"
	awk '{print $2}' "$LISTDIR/load/ip_port.load" > "$LISTDIR/tmp/ip_port_port.$$"
	file2ipset ip_tcp_udp "$LISTDIR/tmp/ip_port_ip.$$"
	file2ipset port_tcp "$LISTDIR/tmp/ip_port_port.$$"
	file2ipset port_udp "$LISTDIR/tmp/ip_port_port.$$"
	rm -f "$LISTDIR/tmp/ip_port_ip.$$" "$LISTDIR/tmp/ip_port_port.$$"
	release_lock "$LOCKFILE_IP" "$FD_IP"
}

use_hook() {
	hook="$HOOKDIR/${0##*/}"
	# shellcheck disable=SC1090
	[ ! -x "$hook" ] || . "$hook"
}

client_prepare_hook() {
	return 0
}

main() {
	log "Загружаем списки IP в ipset'ы"
	set_lock "$LOCKFILE_IP_LOAD" 60 "$FD_IP_LOAD" 1
	client_prepare_hook
	load_ip
	load_ports
	load_ip_ports
	release_lock "$LOCKFILE_IP_LOAD" "$FD_IP_LOAD"
}

use_hook
main "$@"
