#!/bin/bash

# Здесь не стоит делать set -eu
# Здесь не стоит менять названия цепочек
# Переменная forward для обеспечения совместимости кода между CR7 и CR8

. /etc/rc.d/init.d/functions
. /usr/local/Reductor/etc/const
. $CONFIG

declare -a rules
rules=(
	"-p tcp -j REJECT --reject-with tcp-reset"
	"-p udp -j REJECT --reject-with icmp-port-unreachable"
	"-j REJECT --reject-with icmp-proto-unreachable"
)

chain=reductor_maintain
forward=FORWARD

is_first_start() {
	local uptime_seconds
	local ten_minutes=$((10 * 60))
	IFS='. ' read uptime_seconds _ < /proc/uptime
	[ "$uptime_seconds" -lt "$ten_minutes" ]
}

lock() {
	local ip
	if [ "${filter['maintainance']}" = '0' ] && ! $BINDIR/dnsenabled --revisor; then
		$SETCOLOR_FAILURE
		log "НЕ включаем режим обслуживания, так как он выключен в меню."
		log "В течении этого момента, если происходит обновление или перезапуск Carbon Reductor могут быть пропуски фильтрации."
		$SETCOLOR_NORMAL
		return 0
	fi

	if is_first_start; then
		log "НЕ включаем режим обслуживания так как предполагаем включение после перезагрузки или установки."
		return 0
	fi

	iptables -N $chain &>/dev/null || true
	iptables -F $chain
	for rule in "${rules[@]}"; do
		iptables -A $chain $rule
	done
	if $BINDIR/dnsenabled --revisor; then
		log "Полная блокировка доступа в интернет для ${filter['revisor_ip']} на время обслуживания."
		for ip in ${filter['revisor_ip']}; do
			iptables -I $forward -s $ip -j $chain
		done
		# При наличии такого DROP, трафик даже не дойдёт до режима обслуживания
		# Делаем безусловно от конфига, т.к. его могли поменять, а старое правило висит.
		iptables -t raw -D mirror_traffic -j DROP 2>/dev/null || true
	else
		$SETCOLOR_FAILURE
		log "Полная блокировка доступа в интернет на время обслуживания."
		$SETCOLOR_NORMAL
		iptables -I $forward -j $chain
	fi
	mkdir -p $LOCKDIR
	touch $LOCKDIR/maintain.lock
}

unlock() {
	local ip
	is_locked || return 0
	if $BINDIR/dnsenabled --revisor; then
		log "Снятие полной блокировки для ${filter['revisor_ip']}, обслуживание сервера завершено."
	else
		log "Снятие полной блокировки, обслуживание сервера завершено."
	fi
	for _ in {1..3}; do
		iptables -F $chain
		iptables -D $forward -j $chain &>/dev/null || true
		$BINDIR/dnsenabled --revisor || break
		for ip in ${filter['revisor_ip']}; do
			iptables -D $forward -s $ip -j $chain &>/dev/null || true
		done
		# Восстанавливаем удалённый DROP при оптимизации роутинга
		if [ "${filter['optimize_routing']:-0}" = '1' ]; then
			local action
			for action in D D A; do
				iptables -t raw "-$action" mirror_traffic -j DROP &>/dev/null || true
			done
		fi
	done
	rm -f $LOCKDIR/maintain.lock
}

is_locked() {
	local ip
	iptables -C $forward -j $chain &>/dev/null && log "В текущий момент проводится техническое обслуживание сервера, интернет заблокирован всем." && return 0
	if ! $BINDIR/dnsenabled --revisor; then
		log "В текущий момент технического обслуживания не производится."
		return 1
	fi
	for ip in ${filter['revisor_ip']}; do
		if iptables -C $forward -s $ip -j $chain &>/dev/null; then
			log "В текущий момент производится техническое обслуживание для $ip"
			return 0
		else
			log "В текущий момент технического обслуживания для $ip не производится."
		fi
	done
	return 1
}

unlock_delayed() {
	local alarm="Снимаю полную блокировку, несмотря на то, что обслуживание ещё не закончилось. Скорее всего произошло зависание. Нужно разобраться с Carbon Reductor."
	local revisor_locked="Из-за ошибки в процессе рестарта Carbon Reductor не была снята полная блокировка доступа в Интернет для IP ревизора [${filter['revisor_ip']}]. Необходимо подключиться к серверу, убедиться в том, что все в порядке и выполнить команду /usr/local/Reductor/bin/maintain.sh unlock"
	is_locked || return 0
	log "Система находится на обслуживании. Если это продлится ещё 60 секунд, блокировка будет снята."
	sleep 60
	is_locked || return 0
	if $BINDIR/dnsenabled --revisor; then
		alarm="$revisor_locked"
	fi
	log "$alarm"
	log "Администраторам будет послано уведомление об этом."
	echo "$alarm" > $TMPDIR/alarm.unlock
	send_alarm $TMPDIR/alarm.unlock
	$BINDIR/dnsenabled --revisor || unlock
}

main() {
	case "$1" in
		lock | unlock | unlock_delayed | is_locked )
			"$1"
			;;
		* )
			log "Вызвана неизвестная команда: $*"
			exit 1
	esac
}

main "$@"
