#!/usr/bin/env bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090

set -eu

remove_private_networks() {
	egrep -v "^(0.*|1(27|92[.]168|0[.]|72[.](1[6-9]|2[0-9]3[01]))[0-9.]*)(/[0-9]{1,2})?$"
}

maskify() {
	sed -E "s|^$ip_regex$|&/32|g"
}

ip_block() {
	local input="$1"
	local output="$2"
	log "Создаём список IP-хостов: $output.ip"
	egrep "^$ip_regex$" < $input > $output.ip || true
	log "Создаём список IP-подсетей: $output.subnet"
	egrep "^$ipmask_regex$" < $input > $output.subnet || true
}

main() {
	local prefix="$1"
	local input="$LISTDIR/tmp/$prefix.all"
	local output="$LISTDIR/tmp/$prefix.normal"
	if [ "$prefix" == "${prefix//6/}" ]; then
		# shellcheck disable=SC2154
		if [ "$prefix" = 'ip_block' ]; then
			# Для ip_block output отсутствует, есть $output.ip, $output.subnet
			ip_block "$input" "$output"
		else
			egrep "^$ip_or_ipmask_regex" < $input > $output.ip || true
		fi
		# for ip_block there's two $output files, both differ from $input.
		if ! cmp --silent "$input" "$output.ip" && [ "$prefix" != 'ip_block' ]; then
			echo "WARNING: bad data in $input"
			diff -U0 "$input" "$output.ip" || true
		fi
		if [ "$prefix" = 'ip_block' ]; then
			remove_private_networks < "$output.ip" > "$output.tmp" || true
			mv -f  "$output.tmp" "$output.ip"
		else
			maskify < $output.ip > $output.subnet
			remove_private_networks < $output.subnet > $output || true
		fi
	else
		# TODO: Private network support for IPv6
		egrep "^[0-9A-Fa-f:/]+$" < $input > $output || true
	fi

}

main "$@"
