#!/bin/bash

. /usr/local/Reductor/etc/const
. $BINDIR/setup_gui_lib.sh

TIP_USER="Обязательно нужно получить квалифицированную электронную подпись, выданную любым аккредитованным удостоверяющим центром, список центров можно просмотреть в документации по адресу: http://docs.carbonsoft.ru/67403809

Для извлечения закрытого ключа из контейнера нужно положить экспортированный с помощью специальной утилиты контейнер pkcs12 в папку /app/reductor$USERDIR с расширением *.pfx.

Более подробно процесс описан в документации http://docs.carbonsoft.ru/68354136 и http://docs.carbonsoft.ru/42827780

Если у вас возникли трудности - $GOTO_SUPPORT
"


if [ -d /app/reductor ]; then
	__USERDIR="/app/reductor$USERDIR"
else
	__USERDIR="$USERDIR"
fi

NO_PFX_FOUND="Не найден сертификат по адресу $__USERDIR
Проверьте, что вы положили сертификат в нужную директорию"

find_pfx() {
	find $USERDIR -type f -name "*.pfx"
}

convert() {
	PFX="$(find_pfx)"
	[ -z "$PFX" -o ! -s "$PFX" ] && terminate "Не удалось найти корректный pfx-контейнер"
	cp -v "$PFX" ${SSLPRIVATEDIR}/p12.pfx || terminate "Не удалось скопировать PFX-контейнер $PFX в $SSLPRIVATEDIR"
	local pem="/gost-ssl/ssl/provider.pem"
	if [ ! -d /cfg ]; then
		local pem_root="$CHROOTDIR/$pem"
		chroot_mount
		chroot ${CHROOTDIR} ${OPENSSL} pkcs12 -in /gost-ssl/ssl/p12.pfx -out "$pem" -nodes -clcerts || terminate "Не удалось извлечь из $PFX закрытый ключ, $GOTO_SUPPORT"
		chroot_umount
	else
		local pem_root="$pem"
		${OPENSSL} pkcs12 -in /gost-ssl/ssl/p12.pfx -out "$pem" -nodes -clcerts || terminate "Не удалось извлечь из $PFX закрытый ключ, $GOTO_SUPPORT"
	fi
	grep -q 'BEGIN PRIVATE KEY' "$pem_root" || terminate "В provider.pem отсутствует закрытый ключ, $GOTO_SUPPORT"
	cp -vf ${SSLPRIVATEDIR}/provider.pem ${USERDIR}/provider.pem
}

check_pfx_exist() {
	find_pfx | grep -q pfx
}

main() {
	show_msg "$TIP_USER"
	check_pfx_exist || terminate "$NO_PFX_FOUND"
	ask_custom "Конвертировать" "Отмена" "Экспортировать закрытый ключ из $(find_pfx)" || exit
	convert || terminate "Не удалось сконвертировать pfx в pem"
	show_msg "Экспорт сертификата прошёл успешно."
}

${1:-main}
