#!/bin/bash

. /usr/local/Reductor/etc/const
. $BINDIR/setup_master_lib.sh
# shellcheck disable=SC1090
. $CONFIG

TITLE_RKN="Настройка подписи запроса для выгрузки реестра с vigruzki.rkn.gov.ru"
declare -a default_hsts_domains=( 'youtube.com' 'vk.com' 'facebook.com' 'rutube.ru' )

registration() {
	[ -d /cfg ] && return 0 # for compatibility with CR7
	insert_module xt_reductor
	show_msg "Регистрация. Регномер: $($BINDIR/reg_util create)"
	company_name="$(ask_company_name)"
	email="$(ask_email_adress "Введите e-mail адрес человека для решения ТЕХНИЧЕСКИХ вопросов (системного администратора) компании" "email")"
	phone="$(ask_phone_number "Введите номер телефона человека для решения ТЕХНИЧЕСКИХ вопросов (системного администратора) компании" "phone")"
	email_bookkeeper="$(ask_email_adress "Введите e-mail адрес человека для решения ФИНАНСОВЫХ вопросов (бухгалтера) компании" "email_bookkeeper")"
	phone_bookkeeper="$(ask_phone_number "Введите номер телефона человека для решения ФИНАНСОВЫХ вопросов (бухгалтера) компании" "phone_bookkeeper")"
	email_director="$(ask_email_adress "Введите e-mail адрес человека для решения ОРГАНИЗАЦИОННЫХ вопросов (директора) компании" "email_director")"
	phone_director="$(ask_phone_number "Введите номер телефона человека для решения ОРГАНИЗАЦИОННЫХ вопросов (директора) компании" "phone_director")"
	export company_name email email_director email_bookkeeper phone phone_director phone_bookkeeper
	$BINDIR/reg_util update
	$BINDIR/reg_util activate
	$BINDIR/registration.sh activation
}

rkn_update() {
	local email_regexp="^[a-z0-9_\+-]+(\.[a-z0-9_\+-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*\.([a-z]{2,4})$"
	# shellcheck disable=SC2154
	if [ -z "${autoupdate['operator']}" ]; then
		autoupdate['operator']="$(ask_text2 "$TITLE_RKN" 'Полное наименование оператора связи' "$(get_prev_value 'company_name')")"
	fi
	# shellcheck disable=SC2154
	autoupdate['inn']="$(ask_text2 "$TITLE_RKN" 'ИНН оператора связи (10 цифр для юридических лиц, 12 цифр для ИП)' "${autoupdate['inn']}")"
	# shellcheck disable=SC2154
	autoupdate['ogrn']="$(ask_text2 "$TITLE_RKN" 'ОГРН оператора связи (13 цифр для юридических лиц, 15 цифр для ИП)' "${autoupdate['ogrn']}")"
	[ -z "${autoupdate['email']}" ] && autoupdate['email']="$(get_prev_value "email")"
	autoupdate['email']="$(ask_text2 "$TITLE_RKN" "Электронный адрес технического специалиста, ответственного за использование механизма получения выгрузки" "${autoupdate['email']}")"
	check_provider_key
}

rkn_settings() {
	show_msg "Перед настройкой обязательно ознакомьтесь с памяткой оператору связи РКН в документации http://docs.carbonsoft.ru/70189074"
	local url_regexp='(http|https)://[^|/"]+'
	case "$(select_rkn_update_method)" in
		pem )
			rkn_update
			autoupdate['skip_sign_request']='0'
			;;
		xml )
			autoupdate['skip_sign_request']='1'
			show_msg "Следить за работоспособностью запроса и подписи вы обязуетесь самостоятельно. За подробностями обратитесь в документацию и техническую поддержку."
			;;
		web )
			autoupdate['own']='1'
			autoupdate['own_url']="$(question_inputbox "Настройки выгрузок единого реестра" "Введите URL c dump.xml на своем сервере" "$url_regexp" "${autoupdate['own_url']}")"
			;;
	esac
	autoupdate['enabled']="1"
	save_and_exit
}

http() {
	filter['forbidden_url']="http://$($BINDIR/dnsenabled --ext-ip)/block.html"
}

dns() {
	filter['dns_ip']="$($BINDIR/dnsenabled --ext-ip)"
	[ -d /cfg ] && return 0 # for compatibility with CR7
	mkdir -p /etc/nginx/ssl/
	if [ ! -f /etc/nginx/ssl/cert.pem ] || [ ! -f /etc/nginx/ssl/cert.key ]; then
		openssl req -new -x509 -days 9999 -nodes -newkey rsa:2048 -out /etc/nginx/ssl/cert.pem -keyout /etc/nginx/ssl/cert.key
	fi
}

default_hsts_domains_write() {
	for domain in "${default_hsts_domains[@]}"; do
		echo "$domain"
	done
}

proxy() {
	filter['activate_https_proxy']='0' # in CR7 case this just will not be saved
	# для начала вообще определяемся с HTTPS-прокси
	have_proxy="$(question_choice "Вы планируете фильтровать https или hsts сайты через proxy-сервер?

	ВНИМАНИЕ! По умолчанию у ваших клиентов будет заблокирован доступ к следующим  ресурсам:

	$(default_hsts_domains_write | sed -e 's/^/- /;')

	Список доменов можно отредактировать. Фильтрация с помощью proxy-сервера потребует проведения работ по установке сертификата центра сертификации Carbon Reductor всем вашим абонентам. Потребуется организовать возможность оповещения абонентов с подробными инструкциями по настройке, например SMS. Подробности по адресу http://docs.carbonsoft.ru/67404201" "0")"
	[ "$have_proxy" = '1' ] || return 0
	# Предлагаем наш прокси-сервер
	have_proxy_dns="$(question_choice 'Отправлять трафик в прокси-сервер с помощью DNS-спуфинга (простой способ)? Иначе можно будет выбрать более сложный способ с BGP.' 1)"
	if [ "$have_proxy_dns" = '1' ]; then
		# shellcheck disable=SC2154
		filter['activate_https_proxy']='1'
		if [ -d /cfg ]; then
			# shellcheck disable=SC2154
			filter['dns_proxy_ip']="$($BINDIR/dnsenabled --ext-ip)"
		else
			# shellcheck disable=SC2154
			filter['dns_proxy_ip']="$(question_inputbox "Интеграция с прокси-серверами" "IP адрес прокси сервера от Carbon Soft" "$ip_regex" "${filter['dns_proxy_ip']}")"
		fi
		# shellcheck disable=SC2154
		filter['dns_proxy_hsts']="$(question_choice "Отправлять в прокси-сервер с помощью DNS только ресурсы использующие HSTS?\n\n$(default_hsts_domains_write | sed -E 's/^/- /; s/$/\\n/')" '1')"
		# shellcheck disable=SC2154
		filter['dns_proxy_url_https']="$(question_choice "Отправлять в прокси-сервер с помощью DNS ВСЕ имена HTTPS-ресурсов, рекомендуемых к блокировке по URL?" '1')"
		if ! [ -e "$LISTDIR/provider/our.domain_hsts" ]; then
			default_hsts_domains_write > "$LISTDIR/provider/our.domain_hsts"
		fi
	fi

	have_proxy_bgp="$(question_choice 'Отправлять трафик в прокси-сервер с помощью BGP? (Потребуется интеграция)' 0)"
	if [ "$have_proxy_bgp" = '1' ]; then
		# shellcheck disable=SC2154
		if [ -d /cfg ]; then
			filter['bgp_proxy_ip']="$($BINDIR/dnsenabled --ext-ip)"
		else
			filter['bgp_proxy_ip']="$(question_choice "IP адрес HTTPS-прокси сервера" "${filter['bgp_proxy_ip']}")"
		fi
		# shellcheck disable=SC2154
		filter['bgp_proxy_hsts']="$(question_choice "Отправлять в прокси-сервер по IP + резолву только ресурсы использующие HSTS?" "1")"
		# shellcheck disable=SC2154
		filter['bgp_proxy_url_https']="$(question_choice "Отправлять в прокси-сервер по IP и резолву HTTPS-ресурсы, рекомендуемых к блокировке по URL?" "1")"
	fi
}

filter_info() {
	s="Для прокрутки документа используйте клавишу PageDown.\n
\n
Сейчас Carbon Reductor сконфигурирован с типовыми настройками, которые подходят для большинства сетей:\n
\n
- Используется схема с анализом зеркалированного трафика.\n
\n
- HTTP фильтруется встроенным модулем с точностью до URL, абоненту отправляется подставной ответ с HTTP-редиректом на страницу блокировки. На веб-сервер отправляется TCP пакет разрыва соединения.\n
- HTTPS фильтруется несколькими способами:\n
* модуль DNS спуфинга, анализирует зеркало UDP53 трафика и отправляет абоненту подставной UDP53-ответ с адресом страницы блокировки;\n
* модуль SNI, анализирует зеркало HTTPS трафика, с точностью до домена по SNI заголовку TLS. Отправляет TCP пакет разрыва соединения абоненту и веб-серверу;\n
* модуль IP-фильтрации, анализирует зеркало HTTPS трафика, при совпадении IP адресов со списком отправляет TCP пакет разрыва соединения абоненту и веб-серверу. Список IP адресов формируется с помощью эвристического модуля резолвинга IP адресов https ресурсов, с учетом часто меняющихся DNS записей;\n
* экспериментальный PROXY-модуль(по умолчанию выключен), позволяющий фильтровать HTTPS и HSTS с точностью до URL, абоненту отправляется подставной UDP53-ответ с IP адресом Carbon Reductor, вместо адреса сервера, попавшего в реестр. Carbon Reductor проксирует HTTPS трафик и анализирует URL, подлежащие фильтрации. При совпадении переадресует на страницу блокировки. Модуль важен для частичной, а не полной, фильтрации youtube.com и vk.com. Модуль требует добавления корневого сертификата Carbon Reductor абонентам в браузер, в список доверенных центров сертификации. Модуль очень ресурсоемкий и может потребовать установки нескольких серверов. По умолчанию выключен.\n
- Фильтрация прочих протоколов. Используется модуль фильтрации по IP спискам и IP+Port спискам. Отправляет TCP пакет разрыва соединения и/или ICMP ответ абоненту и веб-серверу.\n
- Встроенная страница блокировки. Веб-сервер на 80 и 443 портах Carbon Reductor, который по любому запросу отображает страницу блокировки, в соответствии с рекомендациями Роскомнадзора.\n
\n
Также Carbon Reductor поддерживает интеграцию с оборудованием провайдера с помощью модулей интеграции:\n
- Модуль extDNS. Сообщает DNS серверам провайдера список доменных имен, подлежащих фильтрации. Поддерживаются Bind и Unbound.\n
- Модуль extBGP. Позволяет анонсировать ip адреса, подлежащие фильтрации, по BGP/OSPF на маршрутизаторы провайдера.\n
- Модуль extScript. Позволяет написать произвольный скрипт для интеграции с оборудованием, по событиям скачивания и/или изменения реестра.\n
- Опция указания внешней страницы блокировки. Рекомендуется при большой нагрузке на сервер.\n
\n
Также Carbon Reductor поддерживает сквозную фильтрацию.\n
	"
	show_msg "$s"
}

ip_revisor() {
	filter['revisor_ip']="$(ask_revisor_ip)"
}

filter() {
	filter_info
	ip_revisor
	http
	dns
	proxy
	save_and_exit
}

admins() {
	[ -d /cfg ] && return 0 # for compatibility with CR7
	# shellcheck disable=SC2154
	admin['ip']="$(admin_ip)"
	# shellcheck disable=SC2154
	admin['web_ip']="$(admin_web_ip)"
	[ "${admin['web_port']}" = '80' ] && admin['web_port']='8080'
	admin['web_port']="$(admin_web_port)"
	save_and_exit
}

network() {
	[ -d /cfg ] && return 0 # for compatibility with CR7
	local update_network="$(question_choice "Настроить сеть для приёма зеркала трафика с коммутатора или маршрутизатора?" "1")"
	[ "$update_network" = '0' ] || /usr/local/Reductor/bin/setup_network.sh
}

main() {
	[ ! -d /cfg ] && registration
	rkn_settings
	filter
	if [ ! -d /cfg ]; then
		admins
		network
		/etc/init.d/reductor restart
	fi
}

${1:-main}
