#!/bin/bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. $CONFIG
# shellcheck disable=SC1090
. $MENULIB

ask_custom() {
	local question="$1"
	local param_config="$2"
	local defaultno=""
	if [ "$param_config" == "0" ];then
		defaultno="--defaultno"
	fi
	dialog --stdout $defaultno --yes-label "Да" --no-label "Нет" --yesno "\n$question\n\n" 0 0
}

question_choice() {
	local answer=0
	local question="$1"
	local config_value="$2"
	! ask_custom "$question" "$config_value" || answer=1
	echo $answer
}

get_prev_value() {
	local var="$1"
	/usr/local/Reductor/bin/inline_json /var/reg/CRB-Reductor.reg > /tmp/inline_reg
	( . /tmp/inline_reg; echo "${!var}" )
}

dialog_read_master() {
	dialog --title "$1" --ok-label "Далее" --nocancel --stdout --inputbox "$2" "${3:-0}" "${4:-0}" "${5:-}"
}

question_inputbox() {
	local title="$1"
	local question="$2"
	local regex="$3"
	local prev_value="$4"
	local optional="${5:-0}"
	local var

	while ! echo "${var:-}" | egrep -q "$regex"; do
		var="$(dialog_read_master "$title" "$question" 10 80 "$prev_value")"
		if [ "$optional" = '1' ] && [ "$var" = '' ]; then
			break
		fi
	done
	echo "$var"
}

ask_revisor_ip() {
        local optional="1"
        question_inputbox "Настройка фильтрации" "Введите IP адрес ревизора" "$ip_or_ipmask_regex" "${filter['revisor_ip']}" "$optional"
}

ask_text2() {
	local regex="^[^']+$"
	local title="$1"
	local question="$2"
	local prev_value="$3"
	question_inputbox "$title" "$question" "$regex" "$prev_value"
}

ask_email_adress() {
	local email_regexp="^[a-z0-9_\+-]+(\.[a-z0-9_\+-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*\.([a-z]{2,4})$"
	local optional="${2:-0}"
	question_inputbox "Регистрация" "$1" "$email_regexp" "$(get_prev_value "$2")" $optional
}

ask_phone_number() {
	local optional=1
	question_inputbox "Регистрация" "$1" '[0-9]{8,11}' "$(get_prev_value "$2")" "$optional"
}

ask_inn() {
	# shellcheck disable=SC2154
	question_inputbox "Регистрация" "Введите ИНН (10-12 цифр)" '[0-9]{10,12}' "${autoupdate['inn']}"
}

ask_ogrn() {
	question_inputbox "Регистрация" "Введите ОГРН (13-15 цифр)" '[0-9]{13,15}' "${autoupdate['ogrn']}"
}


TIP_CHECK_PROVIDER_KEY="\nОбязательно нужно получить квалифицированную электронную подпись, выданную любым аккредитованным удостоверяющим центром, список центров можно просмотреть в документации по адресу: http://docs.carbonsoft.ru/70189074

Для извлечения закрытого ключа из контейнера нужно положить экспортированный с помощью специальной утилиты контейнер pkcs12 в папку /app/reductor$USERDIR с расширением *.pfx.

Более подробно процесс описан в документации http://docs.carbonsoft.ru/68354136 и http://docs.carbonsoft.ru/42827780

Если у вас возникли трудности - $GOTO_SUPPORT\n\n\n"

check_provider_key() {
	dialog --stdout --yes-label "Проверить сертификат" --no-label "Пропустить" --yesno "$TIP_CHECK_PROVIDER_KEY" 0 0 || return 0
	$BINDIR/pfx2pem.sh || check_provider_key
}

ask_url_redirect() {
	local url_regexp='(http|https)://[^|/"]+'
	question_inputbox "Настройка фильтрации" "URL для редиректа HTTP-запросов" "$url_regexp" "${filter['forbidden_url']}"
}

ask_local_url() {
	local url_regexp='(http|https)://[^|/"]+'
	question_inputbox "Настройка фильтрации" "Локальный URL для редиректа HTTP-запросов" "$url_regexp" "http://$($BINDIR/dnsenabled --ext-ip)/block.html"
}

# for binary compatibility and easy merges with CR7 only
admin_ip() {
	if [ "${admin['ip']}" = '' ]; then
		admin['ip']="$(who am i | awk '{print $5}' | tr -d '()')"
	fi
	# shellcheck disable=SC2154
	question_inputbox "Администрирование" "Перечислите через пробел список IP адресов администраторов, у них будет доступ к веб-интерфейсу" "$ip_regex" "${admin['ip']}"
}

admin_web_ip() {
	if [ "${admin['web_ip']}" = '' ]; then
		admin['web_ip']="$($BINDIR/dnsenabled --ext-ip)"
	fi
	question_inputbox "Администрирование" "IP адрес на котором будет доступен веб-интерфейс" "$ip_regex" "${admin['web_ip']}"
}


admin_web_port() {
	question_inputbox "Администрирование" "Порт на котором будет доступен веб-интерфейс" '[0-9]{1,5}' "${admin['web_port']}"
}

get_ip_list() {
	tmpfile=/tmp/ip_dns.list
	ip -4 -o a | cut -d ' ' -f 7 | grep -v "^127." | grep -v "^169.25.*/32" | cut -d '/' -f1 > $tmpfile
	while read ip; do
		echo -n "$ip '' '' "
	done < $tmpfile
	echo
	rm -f $tmpfile
}

choose_dns_ip() {
	local baseopts="--stdout --output-fd 2 --colors --trim --backtitle 'setup_master' --no-shadow --item-help --cancel-label 'Не знаю' --ok-label 'Выбрать' "
	local NO_WARRANTY="Мы не гарантируем стабильность её работы, поэтому рекомендуем после завершения тестирования установить заглушку на собственном сервере, либо прямо на Carbon Reductor"
	local ip_list
	local ip
	ip_list="$(get_ip_list) $CARBONSOFT_BLACKHOLE 'Заглушка CarbonSoft' 'Рекомендуется использовать только для тестирования'"
	ip="$(eval "dialog $baseopts --menu 'Выберите IP для DNS-спуфинга:' 11 50 11 $ip_list 3>&1 1>&2 2>&3")"
	if [ "$?" != '0' ] || [ "$ip" = 'Далее' ]; then
		ip="$CARBONSOFT_BLACKHOLE"
		show_msg "Так как вы включили DNS-модуль, но не выбрали IP адрес заглушки, то для демонстрации работы фильтрации DNS-трафика будет использоваться заглушка CarbonSoft.\n\n$NO_WARRANTY"
	elif [ "$ip" = "$CARBONSOFT_BLACKHOLE" ]; then
		show_msg "Вы выбрали заглушку CarbonSoft.\n\n$NO_WARRANTY"
	fi
	filter['dns_ip']="$ip"
}

select_rkn_update_method()
{
	local baseopts="--stdout --output-fd 2 --colors --trim --backtitle 'setup_master' --no-shadow --item-help --nocancel --ok-label 'Выбрать' "
	local variants="'pem' 'Генерация запроса и подписи средствами Reductor' '\Zb\Z3Подписывать запрос на сервере Reductor. Экспорт USB ключа или КриптоПро. '\
		'xml' 'Использование готовых запроса и подписи, созданного на Windows' '\Zb\Z3Необходимо копировать запрос и подпись вручную с windows.'\
		'web' 'Скачивание реестра с собственного веб-сервера' '\Zb\Z3На вашем веб-сервере организуйте собственные скрипты выгрузки реестра.'"
	option="$(eval "dialog $baseopts --menu 'Выбор способа подписи и выгрузки единого реестра' 12 75 10 $variants 3>&1 1>&2 2>&3")"
	echo "$option"
}
