#!/bin/bash

. /usr/local/Reductor/etc/const

TMPFILE=$TMPDIR/hwinfo.tmp
LSPCI=$TMPDIR/lspci.tmp

add_lspci() {
	if [ ! -x /sbin/lspci ]; then
		yum -y install pciutils
	fi
}

grab_data() {
	for interface in /sys/class/net/*/brport/bridge; do
		if [ "$interface" = '/sys/class/net/*/brport/bridge' ]; then
			break
		fi
		bridge="$(readlink $interface | sed 's/.*\///g')"
		eth="$(cut -d / -f5 <<< $interface)"
		echo ${bridge#br} $eth
	done | sort -n | sed 's/^/br/' > $TMPFILE
	lspci > $LSPCI
}

if_ip() {
	ip -4 a show $1 | egrep -o $ip_regex/[0-9]+
}

bridges() {
	echo
	echo "# bridges"
	echo
	for bridge in $(cut -d ' ' -f1 $TMPFILE | uniq); do
		echo $bridge "$(if_ip $bridge)"
		for interface in $(grep -w $bridge $TMPFILE | cut -d ' ' -f2); do
			echo "- $interface $(if_ip $interface)"
		done
	done
}

devices() {
	echo
	echo "# devices"
	echo
	for eth in $(ip link | egrep -o "$device_regex" | sort -u); do
		for id in $(ethtool -i $eth | grep bus-info | sed 's/.*0000://'); do
			is_vlan=0
			if ethtool -i $eth | grep -q '802.1Q VLAN Support'; then
				is_vlan=1
			fi
			if [ "$is_vlan" = 0 ]; then
				rx_buf="$(ethtool -g $eth | tac | grep RX: | egrep -o [0-9]+ | tr '\n' '/'| sed 's|/$||g')"
				echo "$eth: $(grep -w $id $LSPCI) rx: $rx_buf"
			else
				echo "$eth: VLAN interface. rx: default"
			fi
		done
	done
}

_uptime() {
	echo
	echo "# uptime"
	echo
	uptime
}

cpu() {
	echo
	echo "# cpu"
	echo
	echo -e "processor\t: $(nproc)"
	grep 'model name' /proc/cpuinfo | sort | uniq -c
	lscpu
	lscpu -e
}

interrupts() {
	echo
	echo "# interrupts"
	echo
	egrep "$device_regex" /proc/interrupts
	echo
	echo "# softirq"
	echo
	grep NET_RX /proc/softirqs
}

config() {
	echo
	echo "# config"
	echo
	local useless='^(proc|monitoring|misc.diagnostic)'
	egrep "'[01]'" $CONFIG | tr -d "']" | tr '[' '.' | sed 's/=/ = /g' | sort -u | egrep -v "$useless"
}

branch() {
	echo
	echo "# branch"
	echo
	if [ -f /branch ]; then
		cat /branch
	else
		cat /usr/local/Reductor/etc/branch
	fi
}

version() {
	echo
	if [ -f /version ]; then
		echo "# version_used"
		echo
		cat /version
		return 0
	fi
	echo "# version_rpm"
	echo
	rpm -q reductor | tr -d '[a-z-]' | cut -d '.' -f1
	echo
	echo "# version_used"
	echo
	cat /usr/local/Reductor/etc/version
	echo
	echo "# version_web_rpm"
	echo
	rpm -q reductor_web | cut -d '-' -f2,3 | cut -d '.' -f 1,2,3 | tr -d '.-'
}

branch_and_version() {
	branch
	version
}

modules_state() {
	for module in xt_reductor xt_dnsmatch xt_snimatch; do
		file=/proc/net/${module/x/ip}/block_list
		[ -f $file ] || continue
		echo
		echo "# $module"
		echo
		cat $file
	done
}

list_size_print() {
	while read size list; do
		printf "%-40s %s\n" $list $size
	done
}

lists_size() {
	echo
	echo "# Размеры списков"
	echo
	find $LISTDIR/{load,rkn,provider,carbonsoft,resolver} -type f -exec wc -l {} \; | sed "s|$LISTDIR/||g" | list_size_print
}

add_info() {
	echo
	echo "# Информация о провайдере"
	echo
	python -c "import json; print json.dumps(json.load(open('/var/reg/CRB-Reductor.reg')), indent=4, ensure_ascii=False).encode('utf-8')"
}

dfh() {
	[ -d /cfg ] && return 0
	echo
	echo "# Свободное место на файловых системах"
	echo
	df -h
}

centos() {
	echo
	echo "# Версия CentOS"
	echo
	cat /etc/centos-release
	echo
	rpm -q iptables
}

netdevtop() {
	local bin=${BINDIR}/netdevtop.py
	echo
	echo "# Статистика сетевых устройств"
	echo
	[ -d /cfg ] && bin="${bin}c"
	ITERATIONS=1 SCRIPT=1 python "$bin"
}

iptables_forward() {
	echo
	echo "# iptables FORWARD"
	echo
	iptables -nvL FORWARD
	echo
	if iptables-save | grep -m1 -q reductor_forward; then
		iptables -nvL reductor_forward
	fi
	echo
}

server_info_show() {
	echo
	echo "# Используемое оборудование и его настройки"
	echo
	server-info show
	echo
}

server_info_rate() {
	echo
	echo "# Оценка используемого оборудования по шкале от 1 до 10"
	echo
	server-info rate
	echo
}

main() {
	add_info
	add_lspci
	grab_data
	bridges
	devices
	_uptime
	cpu
	interrupts
	config
	branch_and_version
	modules_state
	lists_size
	dfh
	centos
	netdevtop
	iptables_forward
	if [ -d /cfg ]; then
		server_info_rate
		server_info_show
	fi
}

"${@:-main}" | sed -e 's/^[^#$]/    &/g'
