#!/usr/bin/env bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. $CONFIG

set -eu

maskify() {
	sed -E "s|^$ip_regex$|&/32|g"
}

_join() {
	local join=join
	[ "$(uname)" = 'Linux' ] || join=gjoin
	LC_ALL='' LC_CTYPE='' LANG='' $join -t '	' -a 1 -v 1 -j 1 "$@"
}

ip_subnet_block() {
	local output="$1"
	local whitelist="$2"
	local aggregate="$3"
	maskify < "$output.ip" > "$output.ip_masked"
	cat "$output.ip_masked" "$output.subnet" | "$aggregate" -q > "$output.ip_aggregated"
	${BINDIR}/sort.sh "$output.ip_aggregated"
	_join "$output.ip_aggregated" "$whitelist" > "${output//$prefix/ip_subnet_block}"
}

__ip_block() {
	local subnet
	local output="$1"
	local whitelist="$2"
	if [ ! -d /cfg ]; then # CR7
		set +eu
		. ${VENVBIN}/activate # required for subnet2iplist
		set -eu
	fi
	while read subnet; do
		${BINDIR}/subnet2iplist "$subnet"
	done < "$output.subnet" > "$output.subnet_unfolded"
	cat "$output.subnet_unfolded" "$output.ip" > "$output.ip_only"
	${BINDIR}/sort.sh "$output.ip_only"
	_join "$output.ip_only" "$whitelist" > "$output"
}

ip_block() {
	local input="$1"
	local output="$2"
	local whitelist="$3"
	local aggregate="$4"

	${BINDIR}/sort.sh "$whitelist"
	for type in ip subnet; do
		${BINDIR}/sort.sh "$input.$type"
		_join -t '	' -a 1 -v 1 -j 1 "$input.$type" "$whitelist" > "$output.$type"
	done
	ip_subnet_block "$output" "$whitelist" "$aggregate"
	__ip_block "$output" "$whitelist"
}

main() {
	local prefix="$1"
	local input="$LISTDIR/tmp/$prefix.normal"
	local whitelist="$LISTDIR/tmp/${prefix}_whitelist.all"
	local output="${input%.*}.load"
	local aggregate="aggregate"

	[ ! -d /cfg ] && aggregate="$BINDIR/aggregate"
	if [ "$prefix" = 'ip_block' ]; then
		ip_block "$input" "$output" "$whitelist" "$aggregate"
		return
	fi
	${BINDIR}/sort.sh "$input"
	${BINDIR}/sort.sh "$whitelist"
	_join -t '	' -a 1 -v 1 -j 1 "$input" "$whitelist" > "$output"
	if [ "$prefix" = 'ip_https' ] || [ "$prefix" = 'ip_https_plus' ]; then
		"$aggregate" -q < "$output" > "$output.aggregated"
		cp -a "$output.aggregated" "$output"
	fi
}

main "$@"
