#!/bin/bash

. /usr/local/Reductor/etc/const
. $CONFIG

terminate() {
	echo $@
	exit 1
}

MAINDIR=${MAINDIR:-/usr/local/Reductor/}
USERDIR=${USERDIR:-$MAINDIR/userinfo/}
CACHEDIR=${CACHEDIR:-$MAINDIR/cache/}
SSLDIR=${SSLDIR:-$MAINDIR/gost-ssl/}
LOCKDIR="${LOCKDIR:-/var/lock/reductor/}"

LOCKFILE=$LOCKDIR/autoupdate.lock
LOCKTIME=60
LOCKFATAL=1
FD=17

if ! type set_lock &>/dev/null || ! type release_lock &>/dev/null; then
	set_lock() { echo "В режиме восстановления нет возможности повесить блокировку."; }
	release_lock() { echo "В режиме восстановления нет возможности повесить блокировку."; }
fi

old_version=$(</$MAINDIR/etc/version)
[ -z "$old_version" ] && old_version=100000

prepare() {
	if [ -d /usr/local/Reductor ]; then
		return
	fi
	return 0
}

backup() {
	if [ -d /usr/local/old_Reductor.100710 ]; then
		mv /usr/local/old_Reductor.100710 /usr/local/old_Reductor.100710.$(date +%s)
	fi
	if [ "$old_version" = '100710' ]; then
		mv $MAINDIR /usr/local/old_Reductor.$old_version.$(date +%s)
	fi
	if [ -d $MAINDIR ]; then
		mv $MAINDIR /usr/local/old_Reductor.$old_version
	fi
}

update() {
	local rpm="${1:-/tmp/reductor.rpm}"
	if ! yum -y install "$rpm"; then
		if ! yum -y reinstall "$rpm"; then
			mv $MAINDIR /usr/local/bad_Reductor.$$
			mv /usr/local/old_Reductor.$old_version $MAINDIR
			service reductor restart
			echo "Неудачное обновление"
			exit 1
		fi
	fi
}

have_files_in_directory() {
	[ "$(find $1 -maxdepth 1 ! -empty -type f | wc -l)" -gt 0 ]
}

restore() {
	local OLDDIR=/usr/local/old_Reductor.$old_version/
	local OLDPHPDIR=$OLDDIR/reductor_container/gost-ssl/php/
	cp -av $OLDDIR/userinfo/*.pem $OLDDIR/userinfo/*.pfx $USERDIR/
	set -x
	cp -a $OLDDIR/lists/* $MAINDIR/lists/
	cp -a $OLDDIR/cache/* $CACHEDIR/
	set +x
	for file in dump.xml register.zip request{,_second}.xml{,.sign}; do
		if [ -f $OLDPHPDIR/$file ]; then
			cp -av $OLDPHPDIR/$file $SSLDIR/php/
		fi
	done
	if [ -f $OLDDIR/userinfo/config_second ]; then
		cp -v $OLDDIR/userinfo/config_second $USERDIR/config_second
	fi

	mkdir -p $USERDIR/hooks/ $USERDIR/backups/

	old_backups=/usr/local/old_Reductor.$old_version/userinfo/backups/
	if have_files_in_directory $old_backups; then
		set -x
		rm -rf $USERDIR/backups
		cp -a $old_backups $USERDIR/backups
		set +x
	fi

	old_hooks=/usr/local/old_Reductor.$old_version/userinfo/hooks/
	if have_files_in_directory /usr/local/old_Reductor.$old_version/userinfo/hooks/; then
		cp -av $old_hooks/* $USERDIR/hooks/
	fi
}

merge_config() {
        local CONFIG=$USERDIR/config
        local OLDCFG=/usr/local/old_Reductor.$old_version/userinfo/config
        local TMPCONFIG=/tmp/out.cfg
        
        . $CONFIG

        while IFS== read -r var val; do
                [[ "$var" == \#* ]] && echo $var $val && continue
                [ "$var" = '' ] && echo && continue
                [[ "$var" = declare* ]] && echo $var $val && continue
                [[ "$var" != *widget* ]] && . $OLDCFG
                echo "$var='${!var}'"
                . $CONFIG
        done < $CONFIG > $TMPCONFIG
	if [ -s $TMPCONFIG ]; then
        	mv -f $CONFIG $USERDIR/backups/config_$(date +%H.%M.%S_%m-%Y)
		mv -f $TMPCONFIG $CONFIG
	fi
}

run_hooks() {
	for hook in $CONTRIB/hooks/*; do
		if [ -x $hook ]; then
			echo Выполняется хук $hook
			$hook
		fi
	done
}

start_work() {
	/etc/init.d/reductor restart
}

main() {
	set_lock $LOCKFILE $LOCKTIME $FD $LOCKFATAL
	prepare || terminate "prepare failed"
	backup || terminate "backup failed"
	if ! update $@; then
		sleep 5
		terminate "update failed" 
	fi
	restore || terminate "restore failed"
	merge_config || terminate "merge_config failed"
	run_hooks || terminate "run_hooks failed"
	start_work || terminate "start_work failed"
	release_lock $LOCKFILE $FD
}

main $@
